from geometry_msgs.msg import Point, Pose
from trajectory_msgs.msg import JointTrajectory, JointTrajectoryPoint
from builtin_interfaces.msg import Duration
import rclpy
from rclpy.node import Node

from hexapod_python.ik import ik, interpolate_pose, standing_pose, shutdown_pose

import time


class JTCPublisher(Node):
    def __init__(self):
            super().__init__('jtc_publisher')

            self.leg_groups_ = [ "e1", "e2", "e3", "e4", "e5", "e6"]
            self.publisher_ = {}

            for leg in self.leg_groups_:            
                controller_topic = f"/{leg}_group_controller/joint_trajectory"
                self.publisher_[leg] = self.create_publisher(JointTrajectory, controller_topic , 10)


            # wait for the controller to subscribe
            print("Waiting for subscribers")
            for tag in self.leg_groups_:
                while self.count_subscribers(f"/{leg}_group_controller/joint_trajectory") == 0:                    
                    time.sleep(0.1)

            ik_values = [ ik(pose) for pose in interpolate_pose(shutdown_pose, standing_pose, 150) + interpolate_pose(standing_pose, shutdown_pose, 150)]
            print(ik_values)
            for leg in self.leg_groups_:
                t = 0
                joint_trajectory_points = []
                for i in ik_values:
                    joint_trajectory_points.append(
                        JointTrajectoryPoint(
                            positions = i[leg],
                            time_from_start = Duration(nanosec = t%50 * 20000000 , sec = int(t/50))
                        )
                    )
                    t += 1


                print(joint_trajectory_points)
                group_num = leg[1:]
                trajectory = JointTrajectory(
                    joint_names = [ f"c{group_num}_joint", f"f{group_num}_joint", f"t{group_num}_joint" ],
                    points = joint_trajectory_points
                )
                self.publisher_[leg].publish(trajectory)

def main(args = None):
    rclpy.init(args=args)

    jtc_publisher = JTCPublisher()
    rclpy.spin(jtc_publisher)
    jtc_publisher.destroy_node()
    rclpy.shutdown()
    